/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "OutgoingMessagesViewerModule.h"

#include <Core/MessageBroker/MessageBroker.h>

#include <QTableWidgetItem>

#include "Modules/SkywardHubStrings.h"
#include "ui_OutgoingMessagesViewerModule.h"

OutgoingMessagesViewerModule::OutgoingMessagesViewerModule()
    : Module(ModuleId::OUTGOING_MESSAGES_VIEWER),
      ui(new Ui::OutgoingMessagesViewerModule)
{
    ui->setupUi(this);

    MessageBroker::getInstance().subscribe(
        Filter::fromString(SkywardHubStrings::logCommandsTopic), this,
        [this](const Message& message, const Filter& filter)
        { addMsgSent(message); });

    MessageBroker::getInstance().subscribe(
        Filter::fromString(SkywardHubStrings::mavlink_received_msg_ACK_topic),
        this,
        [this](const Message& message, const Filter& filter)
        { handleAck(message); });
    MessageBroker::getInstance().subscribe(
        Filter::fromString(SkywardHubStrings::mavlink_received_msg_NACK_topic),
        this,
        [this](const Message& message, const Filter& filter)
        { handleNack(message); });
    MessageBroker::getInstance().subscribe(
        Filter::fromString(SkywardHubStrings::mavlink_received_msg_WACK_topic),
        this,
        [this](const Message& message, const Filter& filter)
        { handleWack(message); });
}

OutgoingMessagesViewerModule::~OutgoingMessagesViewerModule() { delete ui; }

void OutgoingMessagesViewerModule::addMsgSent(const Message& msg)
{
    int row    = updateVerticalHeaders(msg);
    int column = 0;

    QString msgName           = computeMsgName(msg);
    QTableWidgetItem* newItem = new QTableWidgetItem(msgName);
    newItem->setBackground(QBrush(QColor(80, 80, 80)));
    newItem->setForeground(QBrush(QColor(0, 0, 0)));
    ui->tableWidget->setItem(row, column, newItem);
    registerMessage(newItem, msg);
}

QString OutgoingMessagesViewerModule::computeMsgName(const Message& msg)
{
    return msg.getField("name").toString();
}

void OutgoingMessagesViewerModule::handleAck(const Message& ack)
{
    for (int i = 0; i < messages.count(); i++)
    {
        Message msg = messages[i].getMsg();

        // Color the message the ack is for by checking the message id and
        // sequence number
        if (msg.getField("message_id") == ack.getField("recv_msgid"))
            if (msg.getField("sequence_number") == ack.getField("seq_ack"))
                messages[i].getItem()->setBackground(QBrush(QColor(0, 255, 0)));
    }
}

void OutgoingMessagesViewerModule::handleNack(const Message& nack)
{
    for (int i = 0; i < messages.count(); i++)
    {
        Message msg = messages[i].getMsg();

        // Color the message the nack is for by checking the message id and
        // sequence number
        if (msg.getField("message_id") == nack.getField("recv_msgid"))
            if (msg.getField("sequence_number") == nack.getField("seq_ack"))
                messages[i].getItem()->setBackground(QBrush(QColor(255, 0, 0)));
    }
}

void OutgoingMessagesViewerModule::handleWack(const Message& wack)
{
    for (int i = 0; i < messages.count(); i++)
    {
        Message msg = messages[i].getMsg();

        // Color the message the wack is for by checking the message id and
        // sequence number
        if (msg.getField("message_id") == wack.getField("recv_msgid"))
            if (msg.getField("sequence_number") == wack.getField("seq_ack"))
                messages[i].getItem()->setBackground(
                    QBrush(QColor(255, 255, 255)));
    }
}

int OutgoingMessagesViewerModule::updateVerticalHeaders(const Message& msg)
{
    int row = 0;  // Insert on top
    QTime time(0, 0, 0, 0);
    time = time.addMSecs(msg.getField("timestamp").getInteger());

    QString headerTxt = time.toString("hh.mm.ss (zzz)");
    ui->tableWidget->insertRow(row);
    QTableWidgetItem* header = new QTableWidgetItem(headerTxt);
    ui->tableWidget->setVerticalHeaderItem(row, header);

    int rowCount = ui->tableWidget->rowCount();
    if (rowCount > maxSize && rowCount > 0)
    {
        ui->tableWidget->removeRow(rowCount - 1);
    }

    return row;
}

void OutgoingMessagesViewerModule::registerMessage(QTableWidgetItem* item,
                                                   const Message& msg)
{
    messages.prepend(MessageLog(item, msg));
    if (messages.count() > maxSize)
    {
        messages.removeLast();
    }
}

// ____________________ MESSAGE LOG CLASS _______________________________

MessageLog::MessageLog(QTableWidgetItem* item, const Message& msg)
{
    setItem(item);
    setMsg(msg);
}

QTableWidgetItem* MessageLog::getItem() const { return item; }

void MessageLog::setItem(QTableWidgetItem* value) { item = value; }

Message MessageLog::getMsg() const { return msg; }

void MessageLog::setMsg(const Message& value) { msg = value; }
